#version 120

//#define CROSS_PROCESS //Opposite of desaturation, makes everything more vibrant and saturated.
#define HEAT_REFRACTION 1.00 //How much the screen jiggles around in the nether, or when in lava [0.00 0.25 0.50 0.75 1.00 1.25 1.50 1.75 2.00 2.25 2.50 2.75 3.00 3.25 3.50 3.75 4.00 4.25 4.50 4.75 5.00]

#define INFINITE_OCEANS //Simulates water out to the horizon instead of just your render distance.
#define SEA_LEVEL 63 //Sea level for infinite oceans. Change this if you use custom worldgen. [0 1 2 3 4 5 6 7 8 9 10 11 12 13 14 15 16 17 18 19 20 21 22 23 24 25 26 27 28 29 30 31 32 33 34 35 36 37 38 39 40 41 42 43 44 45 46 47 48 49 50 51 52 53 54 55 56 57 58 59 60 61 62 63 64 65 66 67 68 69 70 71 72 73 74 75 76 77 78 79 80 81 82 83 84 85 86 87 88 89 90 91 92 93 94 95 96 97 98 99 100 101 102 103 104 105 106 107 108 109 110 111 112 113 114 115 116 117 118 119 120 121 122 123 124 125 126 127 128 129 130 131 132 133 134 135 136 137 138 139 140 141 142 143 144 145 146 147 148 149 150 151 152 153 154 155 156 157 158 159 160 161 162 163 164 165 166 167 168 169 170 171 172 173 174 175 176 177 178 179 180 181 182 183 184 185 186 187 188 189 190 191 192 193 194 195 196 197 198 199 200 201 202 203 204 205 206 207 208 209 210 211 212 213 214 215 216 217 218 219 220 221 222 223 224 225 226 227 228 229 230 231 232 233 234 235 236 237 238 239 240 241 242 243 244 245 246 247 248 249 250 251 252 253 254 255 256]

uniform float blindness;
uniform float day;
uniform float rainStrength;
uniform float wetness;
uniform int isEyeInWater;
uniform ivec2 eyeBrightnessSmooth;
uniform mat4 gbufferProjectionInverse;
uniform sampler2D depthtex0;
uniform sampler2D gaux1;
uniform sampler2D gcolor;
uniform sampler2D gnormal;
uniform vec3 actualCameraPosition;
uniform vec3 upPosNorm;

varying vec2 texcoord;

const float actualSeaLevel = SEA_LEVEL - 0.1111111111111111; //water source blocks are 8/9'ths of a block tall, so SEA_LEVEL - 1/9.

const float lavaOverlayResolution                     = 24.0;

//Overworld cloud colors are multi-dimensional because they're used by /lib/fastDrawClouds.glsl for end portal blocks.

#ifdef CROSS_PROCESS
#else
#endif

const vec3 sunsetColorForSky = vec3(7.2, 6.7, 6.2); //Subtract 6.0 from this to get the color of the horizon at sunset.

const vec3 waterScatterColorWhenSunny   = vec3(0.05,    0.4, 0.5);
const vec3 waterScatterColorWhenRaining = vec3(0.09375, 0.1, 0.15);

//simpler algorithm for the special case where distance = infinity (for infinite oceans)
vec3 calcUnderwaterFogColorInfinity(float brightness) {
	return mix(waterScatterColorWhenSunny, waterScatterColorWhenRaining, max(rainStrength, wetness)) * (brightness * day);
}

void main() {
	vec3 color = texture2D(gcolor, texcoord).rgb;
	vec3 normal = texture2D(gnormal, texcoord).xyz * 2.0 - 1.0;
	vec3 aux = texture2D(gaux1, texcoord).rgb;

	float oceanFlag = 1.0;

	//pre-processing for infinite oceans
	#ifdef INFINITE_OCEANS
		float depth = texture2D(depthtex0, texcoord).r;
		if (depth == 1.0) { //sky
			vec3 screenPos = vec3(texcoord, depth);
			vec4 tmp = gbufferProjectionInverse * vec4(screenPos * 2.0 - 1.0, 1.0);
			bool top = dot(tmp.xyz, upPosNorm) > 0.0; //don't care about dot product being normalized
			bool aboveSeaLevel = actualCameraPosition.y > actualSeaLevel;

			if (top) {
				if (!aboveSeaLevel && isEyeInWater == 1) {
					normal = vec3(0.0, -1.0, 0.0);
					aux = vec3(0.0, 1.0, 0.1);
					oceanFlag = 0.0;
				}
			}
			else {
				if (aboveSeaLevel) {
					color = calcUnderwaterFogColorInfinity(0.9384765625 /* (31 / 32) ^ 2 */) * (1.0 - blindness);
					normal = vec3(0.0, 1.0, 0.0);
					aux = vec3(0.0, 0.96875, 0.1);
					oceanFlag = 0.0;
				}
				else if (isEyeInWater == 1) color = calcUnderwaterFogColorInfinity(eyeBrightnessSmooth.y / 240.0) * (1.0 - blindness);
			}
		}
	#endif

/* DRAWBUFFERS:024 */
	gl_FragData[0] = vec4(color, 1.0); //gcolor
	gl_FragData[1] = vec4(normal * 0.5 + 0.5, 1.0); //gnormal
	gl_FragData[2] = vec4(aux, oceanFlag); //gaux1
}